@@ -518,8 +518,17 @@ Layout::GenericLayout *Corona::layout(QString name) const
 }
 
 QRegion Corona::availableScreenRegion(int id) const
-{
-    return availableScreenRegionWithCriteria(id);
+{   
+    //! ignore modes are added in order for notifications to be placed
+    //! in better positioning and not overlap with sidebars or usually hidden views
+    QList<Types::Visibility> ignoremodes({Latte::Types::AutoHide,
+                                          Latte::Types::SidebarOnDemand,
+                                          Latte::Types::SidebarAutoHide});
+
+
+    return availableScreenRegionWithCriteria(id,
+                                             QString(),
+                                             ignoremodes);
 }
 
 QRegion Corona::availableScreenRegionWithCriteria(int id,
@@ -713,7 +722,15 @@ QRegion Corona::availableScreenRegionWithCriteria(int id,
 
 QRect Corona::availableScreenRect(int id) const
 {
-    return availableScreenRectWithCriteria(id);
+    //! ignore modes are added in order for notifications to be placed
+    //! in better positioning and not overlap with sidebars or usually hidden views
+    QList<Types::Visibility> ignoremodes({Latte::Types::AutoHide,
+                                          Latte::Types::SidebarOnDemand,
+                                          Latte::Types::SidebarAutoHide});
+
+    return availableScreenRectWithCriteria(id,
+                                           QString(),
+                                           ignoremodes);
 }
 
 QRect Corona::availableScreenRectWithCriteria(int id,
@@ -1161,9 +1178,16 @@ QStringList Corona::viewTemplatesData()
 
 void Corona::addView(const uint &containmentId, const QString &templateId)
 {
-    auto view = m_layoutsManager->synchronizer()->viewForContainment((int)containmentId);
-    if (view) {
-        view->newView(templateId);
+    if (containmentId <= 0) {
+        auto currentlayouts = m_layoutsManager->currentLayouts();
+        if (currentlayouts.count() > 0) {
+            currentlayouts[0]->newView(templateId);
+        }
+    } else {
+        auto view = m_layoutsManager->synchronizer()->viewForContainment((int)containmentId);
+        if (view) {
+            view->newView(templateId);
+        }
     }
 }
 
